class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
function RenoiseScriptingTool:__init()    
  renoise.Document.DocumentNode.__init(self) 
  self:add_property("Name", "Untitled Tool")
  self:add_property("Version", "1.0")
end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_version = manifest:property("Version").value

function round(value)
  return math.floor(value + 0.5)
end

local function clear_lines(pattern_track, from, to)
  for i = from, to, 1 do
    pattern_track:line(i):clear()
  end
end

local function process()

  local rs = renoise.song()
  
  if (rs.tracks[rs.selected_track_index].type ~= renoise.Track.TRACK_TYPE_SEQUENCER) then
    -- Track isn't a sequencer track
    renoise.app():show_status(tool_name .. ': Selected track is not a sequencer track. Cannot insert slices.')
    return false
  end
  
  local instrument = rs.selected_instrument
  local samples = instrument.samples
  local num_samples = #samples
  
  if (num_samples < 2) then
    -- Nothing to do!
    renoise.app():show_status(tool_name .. ': Selected instrument does not contain any slices.')
    return false
  end
  
  local num_slices = #samples[1].slice_markers

  if (num_slices < 1) then
    -- Nothing to do!
    renoise.app():show_status(tool_name .. ': Selected instrument does not contain any slices.')
    return false
  end
  
  local sample_frames = samples[1].sample_buffer.number_of_frames
  local sample_lines = samples[1].beat_sync_lines
  local frames_per_line = sample_frames / sample_lines
  
  local track = rs.selected_track
  local pattern = rs.selected_pattern
  local pattern_track = rs.selected_pattern_track
  local pattern_lines = pattern.number_of_lines
  
  local first_line = rs.transport.edit_pos.line
  local last_line = math.min((first_line + sample_lines) - 1, pattern_lines)
  
  -- Clear old note data
  clear_lines(pattern_track, first_line, last_line)
  
  -- Show the delay column
  track.delay_column_visible = true
  
  -- Process slices    
  local note_instrument = rs.selected_instrument_index - 1
  local note_panning = 255
  local note_volume = 255
   
  -- Generate notes
  local line = 0
  local frame = 0

  for i = 1, num_slices, 1 do
  
    local slice_frames = samples[i+1].sample_buffer.number_of_frames
    local slice_lines = slice_frames / frames_per_line
    
    -- Generate note
    local offset = math.floor(line)
    local delay = math.floor((line - offset) * 256)
    local pattern_line = first_line + offset
    
    if (pattern_line > pattern_lines) then
      -- Exceeded pattern length
      renoise.app():show_status(tool_name .. ': Reached end of pattern. Cannot insert any more slices.')
      break
    end
    
    local column = 1
    local note = pattern_track:line(pattern_line).note_columns[column]
    while (not note.is_empty) and (note.note_value < 120) and (column < 13) do
      column = column + 1
      if (column > 12) then
        renoise.app():show_status(tool_name .. ': Reached maximum number of note columns. Some slices could not be inserted.')
      else
        note = pattern_track:line(pattern_line).note_columns[column]
      end
    end

    note.instrument_value = note_instrument
    note.note_value = instrument:sample_mapping(1, i+1).note_range[1]

    note.volume_value = note_volume
    note.panning_value = note_panning
    note.delay_value = delay
    
    if (track.visible_note_columns < column) then
      track.visible_note_columns = math.min(12, column)
    end
    
    -- Increment
    line = line + slice_lines    
  end

end

renoise.tool():add_menu_entry {
  name = "Pattern Editor:Pattern:Slices To Pattern",
  invoke = function()
    process()
  end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Pattern:Slices To Pattern",
  invoke = function()
    process()
  end
}
